import json


class BaseBackend(object):
    """
    BaseBackend must subclassed by custom backends to be used. It only
    define basic utilities to help writing backends
    """

    def __init__(self, **kwargs):
        self.template = None
        self.name = kwargs.pop('name')
        self.data = json.dumps(self.fetch(self.name, **kwargs))

    def fetch(self, method, **kwargs):
        """
        Return a set of data formated information
        """
        meth = getattr(self, method, **kwargs)
        return meth()



"""
Here we test the example_bakends methods
"""

from django.test import TestCase
from django.test.client import Client
from django import template
from django.template import TemplateSyntaxError


class TestGraphList(TestCase):

    def test_response(self):
        """
        Check if the url showing all the graph is responding
        """
        client = Client()
        response = client.get("/grapher/all/")
        self.assertEqual(response.status_code, 200)

        self.assertEqual(len(response.context['object_list']), 6)


class TestTemplateTags(TestCase):

    def test_area(self):
        t = template.Template("{% load graph %} {% graph area %}")
        c = template.Context({"STATIC_URL": "/static/"})
        self.assertIsNot(len(t.render(c)), 0)

    def test_bar_chart(self):
        t = template.Template("{% load graph %} {% graph bar_chart %}")
        c = template.Context({"STATIC_URL": "/static/"})
        self.assertIsNot(len(t.render(c)), 0)

    def test_column_chart(self):
        t = template.Template("{% load graph %} {% graph column_chart %}")
        c = template.Context({"STATIC_URL": "/static/"})
        self.assertIsNot(len(t.render(c)), 0)

    def test_pie_chart(self):
        t = template.Template("{% load graph %} {% graph pie_chart %}")
        c = template.Context({"STATIC_URL": "/static/"})
        self.assertIsNot(len(t.render(c)), 0)

    def test_line_chart(self):
        t = template.Template("{% load graph %} {% graph line_chart %}")
        c = template.Context({"STATIC_URL": "/static/"})
        self.assertIsNot(len(t.render(c)), 0)


class TestGraphPages(TestCase):

    def test_ajax(self):
        client = Client()
        response = self.client.get("/grapher/area/",
                                   HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response._headers['content-type'][1],
                         'application/json')

    def test_area(self):
        client = Client()
        response = client.get("/grapher/area/")
        self.assertEqual(response.status_code, 200)

    def test_bar_chart(self):
        client = Client()
        response = client.get("/grapher/bar_chart/")
        self.assertEqual(response.status_code, 200)

    def test_column_chart(self):
        client = Client()
        response = client.get("/grapher/column_chart/")
        self.assertEqual(response.status_code, 200)

    def test_pie_chart(self):
        client = Client()
        response = client.get("/grapher/pie_chart/")
        self.assertEqual(response.status_code, 200)

    def test_line_chart(self):
        client = Client()
        response = client.get("/grapher/line_chart/")
        self.assertEqual(response.status_code, 200)

    def test_404(self):
        client = Client()
        response = client.get("/grapher/this_chart_does_not_exist/")
        self.assertEqual(response.status_code, 404)



"""
This set of templatetags are used to show graph in any pages of your project.
"""

from django import template
from django.conf import settings
from django.utils.safestring import mark_safe
register = template.Library()


def do_graph_node(parser, token):
    """
    render a GraphNode with the right parameters
    """
    try:
        arguments = token.contents.split()
        method = arguments[1]
        args = arguments[2:]
    except IndexError:
        raise template.TemplateSyntaxError, """
%r tag requires at least 1 argument, the name of the graph you want to render
""".format(token.contents.split()[0])
    return GraphNode(method, *args)


class GraphNode(template.Node):
    def __init__(self, method, *args):
        self.method = template.Variable(method)
        self.args = args
        self. obj = None

    def render(self, context):
        backend = __import__(
            settings.GRAPH_BACKEND,
            fromlist=["Backend"])
        klass = getattr(backend, "Backend")
        try:
            name = self.method.resolve(context)
        except template.VariableDoesNotExist:
            name = unicode(self.method)
        self.obj = klass(name=name,
                         *self.args)

        t = template.loader.get_template(
            'mustachebox/tags/{0}.html'.format(self.obj.template))

        return t.render(
            template.Context(
                {'object': self.obj,
                 "STATIC_URL": context['STATIC_URL']},
                autoescape=context.autoescape))

register.tag('graph', do_graph_node)


@register.filter(name='parse_docstring')
def parse_docstring(value):

    from docutils import core
    publish_args = {'source': value, 'writer_name': 'html4css1'}
    parts = core.publish_parts(**publish_args)
    return mark_safe(parts['fragment'])



from django.conf.urls import patterns, url
from django.views.generic import TemplateView
from views import GraphDetailView, GraphListView

urlpatterns = patterns(
    '',
    url(r'^all/$', GraphListView.as_view()),
    url(r'^template_tag_test/$', TemplateView.as_view(
        template_name="mustachebox/templatetags_test.html")),
    url(r'^(?P<name>[-_\w]+)/$',
        GraphDetailView.as_view(),
        name="graph"
        ),
)



import datetime
import time
import random
from mustachebox.backends import BaseBackend


def time_serie(**kwargs):
    """
    Render a time serie
    """
    resp = {2004: ['2004'],
            2005: ['2005'],
            2006: ['2006'],
            2007: ['2007']}

    for i in range(2):
        for k, v in resp.iteritems():
            v.append(int((random.random() * random.random()) * 10000))
        resp['label'] = ['year', 'sales', 'expenses']
    return resp


class Backend(BaseBackend):
    """
    A backend get data formated as mustache need them.
    - data : contain the data to be printed as graph
    - name : the graph name
    - template_name: the template to use for rendering
    """

    def area(self, **kwargs):
        """
        Render a graph area using google chart.

        The data are formated as follow ::

            {
                2004: ['2004', 7160, 546],
                2005: ['2005', 5654, 5435],
                2006: ['2006', 7656, 6545],
                2007: ['2007', 5435, 6545],
                'label': ['year', 'sales', 'expenses']
            }
        """
        self.template = 'area'
        return time_serie(**kwargs)

    def bar_chart(self, **kwargs):
        """
        Render a barchart graph using google chart.

        The data are formated as follow ::

            {
                2004: ['2004', 7160, 546],
                2005: ['2005', 5654, 5435],
                2006: ['2006', 7656, 6545],
                2007: ['2007', 5435, 6545],
                'label': ['year', 'sales', 'expenses']
            }
        """
        self.template = "barchart"
        return time_serie(**kwargs)

    def column_chart(self, **kwargs):
        """
        Render a barchart graph using google chart.

        The data are formated as follow ::

            {
                2004: ['2004', 7160, 546],
                2005: ['2005', 5654, 5435],
                2006: ['2006', 7656, 6545],
                2007: ['2007', 5435, 6545],
                'label': ['year', 'sales', 'expenses']
            }
        """
        self.template = 'columnchart'
        return time_serie(**kwargs)

    def pie_chart(self, **kwargs):
        """
        Define a generic pie chart using google chart

        The data are formated as follow::

            {'label': ['name', 'count'],
             'records': [
                             ['eat': 12],
                             ['read': 5],
                             ['work': 23],
                             ['sleep': 4],
                             ['wathc TV': 10]
                           ]
            }
        """
        self.template = "pie_chart"

        label = ['name', 'count']
        activities = []
        words = ['eat', 'read', 'work', 'sleep', 'watch TV']
        for elem in words:
            activities.append([elem, int(random.random() * 10)])

        return {'label': label, 'records': activities}

    def line_chart(self, **kwargs):
        """
        Define a generic line chart using D3js

        Data are formated as follow ::

            [
              {'date': 1107151200000,
               'value': 289
              },
              {'date': 1109743200000,
               'value': 766
              }
               <...>
            ]
        """
        response = []
        self.template = "line_chart"
        date = datetime.datetime(2005, 1, 1, 0, 0, 0)
        for i in range(1500):
            date += datetime.timedelta(days=2)
            response.append({
                'date': time.mktime(date.timetuple()) * 1000,
                'value': (i * 10) + int(random.random() * 1000)})
        return response

    def multiserie_linechart(self, **kwargs):
        """
        Define a multi-line graph using D3js

        data are formated as follow ::

            [
              {'date': 1109743200000,
              'value': 453,
              'serie': "serie1"
              },
              {'date': 1107151200000,
               'value': 435,
               'serie': "serie2",
              },
              ...
            ]
        """
        self.template = "multiline_chart"
        response = []
        for a in range(3):  # we render a 3 series chart
            date = datetime.datetime(2005, 1, 1, 0, 0, 0)
            for i in range(200):
                if a % 3 == 0:
                    mult = -10
                elif a % 2 == 0:
                    mult = 1
                else:
                    mult = 10
                date += datetime.timedelta(days=2)
                response.append({
                    'date': time.mktime(date.timetuple()) * 1000,
                    'value': (i * mult) + int(random.random() * 1000),
                    'serie': a})
        return response



from django.views.generic.detail import DetailView
from django.views.generic.list import ListView
from django.conf import settings
from django import http


class GraphListView(ListView):
    """
    This class return all the avalaible graphs in the current backend
    """
    template_name = "mustachebox/all_graph.html"

    def get_queryset(self):
        """
        This method does not return a queryset because there is no
        database in this app.
        But it return a list of avalaible grapg to be displayed.
        """
        backend = __import__(
            settings.GRAPH_BACKEND,
            fromlist=["Backend"])

        klass = getattr(backend, "Backend")
        responses = []
        for k, v in klass.__dict__.iteritems():
            if hasattr(v, '__call__'):
                responses.append({'name': k, 'doc': v.__doc__})
        return responses


class GraphDetailView(DetailView):
    """
    This class represent a graph data.  The data you will use to
    represent a graph may not be part of you application nor your
    project. It can be somethin' commin' from an external API, a nosql
    backend, your cache backend and so one.

    So GraphDetailView implement a "get object" method to retreive the
    data using the backend you have define in your settings.

    Look at mustachebox.backends for an example backend
    """

    def get_object(self):
        backend = __import__(
            settings.GRAPH_BACKEND,
            fromlist=["Backend"])
        klass = getattr(backend, "Backend")
        try:
            obj = klass(**self.kwargs)
        except AttributeError:
            raise http.Http404
        return obj

    def get_context_object_name(self, obj):
        return "object"

    def get_template_names(self):
        return "mustachebox/{0}.html".format(self.object.template)

    def render_to_response(self, context, **response_kwargs):
        if self.request.is_ajax():
            return http.HttpResponse(context['object'].data,
                                     content_type='application/json',
                                     **response_kwargs)
        return super(
            GraphDetailView, self).render_to_response(
                context,
                **response_kwargs)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os

VERSION = "0.2"
REQUIRES = ["django(>=1.3)","docutils"]
here = os.path.abspath(os.path.dirname(__file__))

setup(
    name="mustachebox",
    version=VERSION,
    description="django framework to create charts of data",
    long_description="""MustacheBox is a set of utilities to help you presenting graph of
various data in a Django project. Data can come from whatever source you want :

* Distant API
* Couchdb databases
* Your Django models
* ...

And could be rendered using the js library of your choice : google
charts, d3js, raphael etc... """,
    packages = find_packages(),
    package_data={ 'mustachebox':
                    ['templates/*.html',
                     'templatetags/*',
                     'static/css/*',
                     'static/js/*',
                     'templates/mustachebox/*.html',
                     'templates/mustachebox/tags/*.html',
                     ] },
    author="Yohann Gabory",
    author_email="yohann@gabory.fr",
    url="https://bitbucket.org/boblefrag/mustachebox",
    download_url="https://bitbucket.org/boblefrag/mustachebox/get/mustachebox-0.2.tar.gz",
    requires=REQUIRES,
    classifiers = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    'Programming Language :: Python',
    'Operating System :: OS Independent',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Utilities',
    ],
    include_package_data=True,
    )
